@@ -2,6 +2,7 @@ package bn256
 
 import (
 	"fmt"
+	"math/big"
 )
 
 type gfP [4]uint64
@@ -18,6 +19,23 @@ func newGFp(x int64) (out *gfP) {
 	return out
 }
 
+func newGFpFromBigInt(bigInt *big.Int) *gfP {
+	leftPad32 := func(in []byte) []byte {
+		if len(in) > 32 {
+			panic("input cannot be more than 32 bytes")
+		}
+
+		o := make([]byte, 32)
+		copy(o[32-len(in):], in)
+		return o
+	}
+
+	out := new(gfP)
+	out.Unmarshal(leftPad32(bigInt.Bytes()))
+	montEncode(out, out)
+	return out
+}
+
 func (e *gfP) String() string {
 	return fmt.Sprintf("%16.16x%16.16x%16.16x%16.16x", e[3], e[2], e[1], e[0])
 }
@@ -66,5 +84,15 @@ func (e *gfP) Unmarshal(in []byte) {
 	}
 }
 
+func (e *gfP) BigInt() *big.Int {
+	bigInt := new(big.Int)
+	decoded := new(gfP)
+	montDecode(decoded, e)
+	buf := make([]byte, 32)
+	decoded.Marshal(buf)
+	bigInt.SetBytes(buf)
+	return bigInt
+}
+
 func montEncode(c, a *gfP) { gfpMul(c, a, r2) }
 func montDecode(c, a *gfP) { gfpMul(c, a, &gfP{1}) }
